﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Mono.Upnp;

namespace StatisticsBoxSDK
{
    public class DeviceBrowser
    {
        readonly object mutex = new object();
        const int kTimeout = 5000; //ms
        const string kOurUuid = "fd9c2a49-f4fd-477e-a592-";
        List<Device> m_Devices = new List<Device>();
        void ClientDeviceAdded(object sender, DeviceEventArgs e)
        {
            var mono_device = e.Device.GetDevice();
            if (mono_device.Udn.Contains(kOurUuid))
            {         
                String uniqueDeviceName = mono_device.Udn;
                String friendlyName = mono_device.FriendlyName;
                String presentationURL = mono_device.PresentationUrl;
                String modelName = mono_device.ModelName;               
                String modelNumber = mono_device.ModelNumber;
                String serialNumber = mono_device.SerialNumber;
                Device device = new Device(uniqueDeviceName, friendlyName, presentationURL, modelName, modelNumber, serialNumber);
                lock (mutex)
                {
                    m_Devices.Add(device);
                }  
                /*
                Console.WriteLine("ClientDeviceAdded");
                Console.WriteLine(mono_device.Udn);
                Console.WriteLine(mono_device.Type);
                Console.WriteLine(mono_device.FriendlyName);
                Console.WriteLine(mono_device.ModelName);
                Console.WriteLine(mono_device.PresentationUrl);
                Console.WriteLine("---------");   
                 * */
            }
        }

        void ClientDeviceRemoved(object sender, DeviceEventArgs e)
        {      
            /*
            Console.WriteLine("ClientDeviceRemoved");
            Console.WriteLine(e.Device.Udn);
            */
            lock (mutex)
            {
                m_Devices.RemoveAll(device => device.UniqueDeviceName == e.Device.Udn);
            }            
        }

        public List<Device> FindDevices()
        {
            using (Client m_Client = new Client())
            {
                m_Devices.Clear();
                m_Client.DeviceAdded += ClientDeviceAdded;
                m_Client.DeviceRemoved += ClientDeviceRemoved;
                m_Client.BrowseAll();
                Thread.Sleep(kTimeout);
            }
            lock (mutex)
            {
                List<Device> devices_copy = new List<Device>(m_Devices);
                return devices_copy;
            }
        }

        public void RefreshDevice(ref Device device)
        {
            String udn = device.UniqueDeviceName;
            device = null;
            int n = 0;
            while (device == null)
            {
                using (Client m_Client = new Client())
                {
                    m_Devices.Clear();
                    m_Client.DeviceAdded += ClientDeviceAdded;
                    m_Client.DeviceRemoved += ClientDeviceRemoved;
                    m_Client.BrowseAll();
                    Thread.Sleep(kTimeout);
                }
                lock (mutex)
                {
                    device = m_Devices.FirstOrDefault(dev => dev.UniqueDeviceName == udn);
                }
                if (device == null)
                    Console.WriteLine("Device not found! Retrying... " + n.ToString());  
                n++;                    
            }
            Console.WriteLine("Selected " + device.PresentationURL);  
        }
    }
}
