﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web.Script.Serialization;

namespace StatisticsBoxSDK
{
    public class Point
    {
        public double x;
        public double y;
        override public string ToString()
        {
            return "{" + x.ToString(CultureInfo.InvariantCulture) + "," + y.ToString(CultureInfo.InvariantCulture) + "}";
        }
    }

    public class Line
    {
        public Point[] points = new Point[0];
        override public string ToString()
        {
            string concatenated = string.Join(",",
                          points.Select(x => x.ToString()).ToArray());
            return "{{" + concatenated + "}}";
        }
    }

    public class Circle
    {
        public Point[] points = new Point[2] { new Point(), new Point() };
        override public string ToString()
        {
            string concatenated = string.Join(",",
                          points.Select(x => x.ToString()).ToArray());
            return "{" + concatenated + "}";
        }
    }

    [Serializable]
    public class AlgoParameters
    {
        public Line line = new Line();
        public Circle target_circle = new Circle();
    }

    public class AlgoParametersConverter : JavaScriptConverter
    {

        public override IEnumerable<Type> SupportedTypes
        {
            get { return new[] { typeof(AlgoParameters) }; }
        }

        public override IDictionary<string, object> Serialize(object obj, JavaScriptSerializer serializer)
        {
            throw new NotImplementedException();
        }

        private static Point[] ArrayToPoints(double[][] array)
        {
            if (array == null) return null;
            List<Point> points = new List<Point>();
            foreach (double[] element in array)
            {
                points.Add(new Point { x = element[0], y = element[1] });
            }
            return points.ToArray();
        }

        public override object Deserialize(IDictionary<string, object> dictionary, Type type, JavaScriptSerializer serializer)
        {            
            if (dictionary == null)
                throw new ArgumentNullException("dictionary");

            if (type == typeof(AlgoParameters))
            {
                AlgoParameters algoParameters = new AlgoParameters();
                String lineString = dictionary["line"] as String;
                String circleString = dictionary["circle"] as String;

                lineString = lineString.Replace('{', '[').Replace('}', ']');
                circleString = circleString.Replace('{', '[').Replace('}', ']');

                double[][][] line_array = serializer.Deserialize<double[][][]>( lineString );
                double[][] circle_array = serializer.Deserialize<double[][]>( circleString );

                algoParameters.line = new Line();
                algoParameters.line.points = ArrayToPoints(line_array[0]);

                algoParameters.target_circle = new Circle();
                algoParameters.target_circle.points = ArrayToPoints(circle_array);

                return algoParameters;
 
            }
            return null;
        }
       
    }
}
