﻿using System;
using System.Collections.Generic;
using StatisticsBoxSDK;
using System.Linq;

namespace TestSBSDK
{
    class Program
    {  
        static void DumpObject<T>(T obj)
        {
            System.Console.WriteLine("{0}", obj.GetType().Name);
            obj.GetType().GetFields()
                .ToList()
                .ForEach(pi => System.Console.WriteLine("{0}={1}", pi.Name, pi.GetValue(obj).ToString()));
        }

        static void TestDeviceSettings(Device dev){
            DeviceSettings deviceSettings = dev.GetDeviceSettings();
            Console.WriteLine("-----Got DeviceSettings.");
            DumpObject(deviceSettings);
            DeviceSettings newDeviceSettings = new DeviceSettings();
            newDeviceSettings.datetime.dateTime = new DateTime(2018, 5, 1, 8, 30, 52);
            newDeviceSettings.device_name = "Test n&ame"; //url encode test
            newDeviceSettings.ntp = "ntp.server";
            newDeviceSettings.timezone = "Asia/Yekaterinburg";
            dev.SetDeviceSettings(newDeviceSettings);
            Console.WriteLine("-----Set DeviceSettings.");
            newDeviceSettings = dev.GetDeviceSettings();
            Console.WriteLine("-----Got DeviceSettings.");
            DumpObject(newDeviceSettings);
            dev.SetDeviceSettings(deviceSettings);
        }


        static void TestAlgoParameters(Device dev)
        {
            AlgoParameters algoParameters = dev.GetAlgoParams();
            Console.WriteLine("-----Got AlgoParameters.");
            DumpObject(algoParameters);
            AlgoParameters newAlgoParameters = new AlgoParameters();
            newAlgoParameters.target_circle.points[0].x = 0.5;
            newAlgoParameters.target_circle.points[0].y = 0.5;
            newAlgoParameters.target_circle.points[1].x = 0.5;
            newAlgoParameters.target_circle.points[1].y = 0.8;
            newAlgoParameters.line.points = new Point[] { 
                                new Point { x = 0, y = 0.3 }, new Point { x = 1, y = 0.3 } 
                            };
            dev.SetAlgoParams(newAlgoParameters);
            Console.WriteLine("-----Set AlgoParameters.");
            newAlgoParameters = dev.GetAlgoParams();
            Console.WriteLine("-----Got AlgoParameters.");
            DumpObject(newAlgoParameters);
            dev.SetAlgoParams(algoParameters);
        }

        static void TestHostnameSettings(Device dev)
        {
            HostnameSettings hostnameSettings = dev.GetHostname();
            Console.WriteLine("-----Got HostnameSettings.");
            DumpObject(hostnameSettings);
            HostnameSettings newHostnameSettings = new HostnameSettings();
            newHostnameSettings.hostname = "test";
            dev.SetHostname(newHostnameSettings);
            Console.WriteLine("-----Set HostnameSettings.");
            newHostnameSettings = dev.GetHostname();
            Console.WriteLine("-----Got HostnameSettings.");
            DumpObject(newHostnameSettings);
            dev.SetHostname(hostnameSettings);
        }

        static void TestWebserverSettings(Device dev)
        {
            WebserverSettings webserverSettings = dev.GetWebserverSettings();
            Console.WriteLine("-----Got WebserverSettings.");
            DumpObject(webserverSettings);
            WebserverSettings newWebserverSettings = new WebserverSettings();
            newWebserverSettings.port = 10000;
            newWebserverSettings.protocol = "http";
            dev.SetWebserverSettings(newWebserverSettings);
            Console.WriteLine("-----Set WebserverSettings.");
            newWebserverSettings = dev.GetWebserverSettings();
            Console.WriteLine("-----Got WebserverSettings.");
            DumpObject(newWebserverSettings);
            dev.SetWebserverSettings(webserverSettings);
        }

        static void TestNetSettings(ref Device dev, DeviceBrowser browser)
        {
            NetSettings netSettings = dev.GetNetSettings();
            Console.WriteLine("-----Got NetSettings.");
            DumpObject(netSettings);
            NetSettings newNetSettings = new NetSettings();
            newNetSettings.dhcp_key = "off";
            newNetSettings.dns = "192.168.1.29";
            newNetSettings.gateway = "10.200.0.1";
            newNetSettings.ip_device = "10.200.20.134";
            newNetSettings.mask = "255.255.0.0";
            dev.SetNetSettings(newNetSettings);
            browser.RefreshDevice(ref dev);
            dev.Connect("admin", "admin");
            Console.WriteLine("-----Set NetSettings.");
            newNetSettings = dev.GetNetSettings();
            Console.WriteLine("-----Got NetSettings.");
            DumpObject(newNetSettings);
            netSettings = new NetSettings();
            netSettings.dhcp_key = "on";
            dev.SetNetSettings(netSettings);
            browser.RefreshDevice(ref dev);
            dev.Connect("admin", "admin");
        }

        static void TestReport(Device dev)
        {
            String report = dev.GetReport("xml", new DateTime(2008, 5, 1, 8, 30, 52), DateTime.Now);
            Console.WriteLine("-----Got Report.");
            Console.WriteLine("{0}", report);
        }

        static void TestClearDb(Device dev)
        {
            dev.ClearDb(new DateTime(2008, 5, 1, 8, 30, 52), DateTime.Now);
            Console.WriteLine("-----Cleared DB.");
            String report = dev.GetReport("xml", new DateTime(2008, 5, 1, 8, 30, 52), DateTime.Now);
            Console.WriteLine("-----Got Report.");
            Console.WriteLine("{0}", report);
        }

        static void TestUpdate(Device dev)
        {
            dev.UploadUpdate("C:\\ForVm\\update-3.1---SVN_rev_2584M.trunk---2013-08-08__09_59_29.tar.bz2.asc");
            Console.WriteLine("-----Uploaded update.");
        }
        

        static void Main(string[] args)
        {
            DeviceBrowser browser = new DeviceBrowser();
            {
                List<Device> devs = browser.FindDevices();
                Console.WriteLine("Num Of devs found by browser = {0}\n", devs.Count);
                /*foreach (String host in (new []{"statdev","10.200.20.202"}))
                {
                    devs.Add(Device.CreateFromHost(host));
                }*/
                foreach (Device dev in devs)
                {
                    Console.WriteLine("Operating with device {0}:", dev.PresentationURL);

                    dev.Connect("admin", "admin");
                    Device local_dev = dev;
                    NetSettings netSettings = dev.GetNetSettings();
                    if (netSettings.ip_device == "10.200.11.14")
                    {
                        TestNetSettings(ref local_dev, browser);
                        TestDeviceSettings(local_dev);
                        TestAlgoParameters(local_dev);
                        TestHostnameSettings(local_dev);
                        TestWebserverSettings(local_dev);
                        TestReport(local_dev);
                        //TestClearDb(local_dev);
                        //TestUpdate(local_dev);
                    }    
                }
            }
        }
    }
}
